/*[]------------------------------------------------------------[]*/
/*|                                                              |*/
/*|     Source code for TDUSER32.DLL (for TD32.EXE)              |*/
/*|                                                              |*/
/*|                                                              |*/
/*|     Copyright (c) 1991, 1994 by Borland International        |*/
/*|     All Rights Reserved.                                     |*/
/*|                                                              |*/
/*[]------------------------------------------------------------[]*/

/* Written by Jeffrey J. Peters */
#include <windows.h>
#include <conio.h>
#include <dos.h>
#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include "tduser.h"   // prototypes for our TDVIDEO API

FILE *log = NULL;
int showPoll = 0;
int log_memory = 0;

TDUserInfoBlock *gPtr = 0;

extern "C" void UIStart (unsigned short, HANDLE);
// #define INVOKE_CODE 0x8F00          // Ctrl-Num5
#define INVOKE_CODE 0x1C0A          // Ctrl-Enter

struct modeBlock
{
  UCHAR c;
  char *str;
};

modeBlock mb[] =
{
  {  EP_NONE           ,"EP_NONE"},
  {  EP_CTRLBRK        ,"EP_CTRLBRK"},
  {  EP_TRACE          ,"EP_TRACE"},
  {  EP_BKPT           ,"EP_BKPT"},
  {  EP_TERM           ,"EP_TERM"},
  {  EP_LOADED         ,"EP_LOADED"},
  {  EP_RUNNING        ,"EP_RUNNING"},
  {  EP_SWAPPED        ,"EP_SWAPPED"},
  {  EP_STEP           ,"EP_STEP"},
  {  EP_KEYINTR        ,"EP_KEYINTR"},
  {  EP_SYSBKPT        ,"EP_SYSBKPT"},
  {  EP_GLOBAL         ,"EP_GLOBAL"},
  {  EP_NMI            ,"EP_NMI"},
  {  EP_HARDBKP        ,"EP_HARDBKP"},
  {  EP_EXCEPTION      ,"EP_EXCEPTION"},
  {  EP_FPERROR        ,"EP_FPERROR"},
  {  EP_ALLFROZEN      ,"EP_ALLFROZEN"},
  {  EP_THREADTERM     ,"EP_THREADTERM"},
  {  EP_SIGNAL         ,"EP_SIGNAL"},
  {  EP_NEWPROCESS     ,"EP_NEWPROCESS"},
  {  EP_WINBKPT        ,"EP_WINBKPT"},
  {  EP_EXCEPFIRSTTRY  ,"EP_EXCEPFIRSTTRY"},
  {  EP_EXCEPUNEXPECTED,"EP_EXCEPUNEXPECTED"},
  {  EP_EXCEPTERMINATE ,"EP_EXCEPTERMINATE"},
  {  EP_SYSERROR       ,"EP_SYSERROR"}
};
#define MODESIZE (sizeof(mb)/sizeof(mb[0]))

char *FindMode (UCHAR m)
{
  for (int i=0;i<MODESIZE;i++)
    if (mb[i].c == m)
      return mb[i].str;

  return "Unknown";
}
//--------------------------------------------------------------------------//
unsigned short GetKey (unsigned short mode)
{
  _EAX = 0;
  _AX = mode;
  geninterrupt (0x16);
  return _AX;
}

//--------------------------------------------------------------------------//
#pragma argsused
extern "C" int FAR PASCAL LibMain( HANDLE h, WORD wDataSegment,
                        WORD wHeapSize, LPSTR lpszCmdLine )
/*
   This is a standard Windows DLL LibMain function.
*/
{
    return 1;   // Always succeed LibMain
}
//--------------------------------------------------------------------------//
#pragma argsused
extern "C" int FAR PASCAL WEP ( int bSystemExit )
/*
   This is a standard Windows DLL WEP function.
*/
{
    return 1;  // Unload succeeded
}
//--------------------------------------------------------------------------//
int _WIN UserAPIInit (TDUserInfoBlock *i)
{
  char *p, *opts;


  opts = getenv ("TDUSEROPT");
  if (opts)
  {
    if (strstr (opts, "-p"))
      showPoll = 1;

    if (strstr (opts, "-m"))
      log_memory = 1;
  }

  p = getenv ("TDUSERLOG");
  if (p)
  {
    log = fopen (p, "wt");
  }
  else
  p = getenv ("TDUSER");
  if (p)
  {
    log = fopen (p, "wt");
  }

  gPtr = i;  // save it for later

  if (log)
  {
    fprintf (log, "\nUserAPIInit:  ShowPOLL = %d", showPoll);
    fprintf (log,
              "\n  TDUserInfoBlock contents:  \n"
              "    InfoBlockSize:             %04Xh\n"
              "    APIVersion:                %04Xh\n"
              "    EvalFunc:                 :%08X\n"
              "    pCPU:                     :%08X\n"
              "    p8087:                    :%08X\n"
              "    pProgState:               :%08X\n"
              "    pCurrentThreadID:         :%08Xh\n"
              "    pUserProcessID:           :%08Xh\n"
              "    pScreenSel:               :%08Xh\n",
              (int)i->InfoBlockSize,
              (int)i->APIVersion,
              i->fpEvalFunc, i->pCPU,
              i->p8087, i->pProgState, i->pCurrentThreadID,
              i->pUserProcessID,i->pScreenSel
            );

    fflush (log);
  }
  return 0;  // 0 = success
}
//--------------------------------------------------------------------------//
void _WIN UserAPIDone (void)
{
  if (log)
  {
    fprintf (log, "\nUserAPIDone:");
    fflush (log);
    fclose (log);
    log = NULL;
  }
}
//--------------------------------------------------------------------------//
void _WIN UserAPIModuleLoad (char *modname, long handle)
{
  if (log)
  {
    fprintf (log, "\nUserAPIModuleLoad:   (%08X) %s",
             handle, modname);
    fflush (log);
  }
}
//--------------------------------------------------------------------------//
void _WIN UserAPIModuleUnload (char *modname, long handle)
{
  if (log)
  {
    fprintf (log, "\nUserAPIModuleUnload: (%08X) %s",
             handle, modname);
    fflush (log);
  }
}
//--------------------------------------------------------------------------//
unsigned short _WIN UserAPIKeyPressed (unsigned short bioskeycode)
{
  if (log)
  {
    fprintf (log, "\nUserAPIKeyPressed:   %04X", bioskeycode);
  if (log)
    if (bioskeycode == ((unsigned short)INVOKE_CODE))
      fprintf (log, " (Invoke)");
    fflush (log);
  }

  if (bioskeycode == ((unsigned short)INVOKE_CODE))
  {
    UIStart (*gPtr->pScreenSel, (HANDLE)*gPtr->pUserProcessID);
    return 0;
  }

  return bioskeycode;
}
//--------------------------------------------------------------------------//
unsigned short _WIN UserAPIPollForKey (int biosmode)
{
  if (showPoll)
  if (log)
  {
    fprintf (log, "\nUserAPIPollForKey:   %08X", biosmode);
    fflush (log);
  }
  return -1;
}
//--------------------------------------------------------------------------//
void _WIN UserAPIChildStop (void)
{
  if (log)
  {
    fprintf (log, "\nUserAPIChildStop: Mode: %s", FindMode (*gPtr->pProgState));
    fprintf (log, " Sel:%04X, TID:%04X ",
                  *gPtr->pScreenSel,
                  *gPtr->pCurrentThreadID);
    fflush (log);
  }
}
//--------------------------------------------------------------------------//
void _WIN UserAPIChildStart (void)
{
  if (log)
  {
    fprintf (log, "\nUserAPIChildStart:");
    fflush (log);
  }
}
//--------------------------------------------------------------------------//
int EvalExpr (char *str, char *result)
{
  int ret = 0;

  if (gPtr)
  {
    if (gPtr->fpEvalFunc)
    {
      ret = gPtr->fpEvalFunc (str, result);
    }
  }
  return ret;
}